package com.tourism.common.qcloud.cos.config;

import com.tourism.common.qcloud.cos.wrapper.QcloudCosTemplate;
import com.qcloud.cos.COSClient;
import com.qcloud.cos.ClientConfig;
import com.qcloud.cos.auth.BasicCOSCredentials;
import com.qcloud.cos.auth.COSCredentials;
import com.qcloud.cos.http.HttpProtocol;
import com.qcloud.cos.region.Region;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;

/**
 * common-qcloud-cos块的自动配置引导类。仅当配置项qcloud.cos.enabled为true的时候加载。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@EnableConfigurationProperties(QcloudCosProperties.class)
@ConditionalOnProperty(prefix = "qcloud.cos", name = "enabled")
public class QcloudCosAutoConfiguration {

    /**
     * 将腾讯云COS原生的客户端类封装成bean对象，便于集成，同时也可以灵活使用客户端的所有功能。
     *
     * @param p 属性配置对象。
     * @return 腾讯云COS的原生客户端对象。
     */
    @Bean
    @ConditionalOnMissingBean
    public COSClient qcloudCosClient(QcloudCosProperties p) {
        COSCredentials cred = new BasicCOSCredentials(p.getAccessKey(), p.getSecretKey());
        Region region = new Region(p.getRegion());
        ClientConfig clientConfig = new ClientConfig(region);
        // 这里建议设置使用 https 协议
        // 从 5.6.54 版本开始，默认使用了 https
        clientConfig.setHttpProtocol(HttpProtocol.https);
        COSClient client = new COSClient(cred, clientConfig);
        if (!client.doesBucketExist(p.getBucketName())) {
            client.createBucket(p.getBucketName());
        }
        return client;
    }

    /**
     * 封装的腾讯云COS模板类。
     *
     * @param p 属性配置对象。
     * @param c 腾讯云COS的原生客户端bean对象。
     * @return 腾讯云COS模板的bean对象。
     */
    @Bean
    @ConditionalOnMissingBean
    public QcloudCosTemplate qcloudCosTemplate(QcloudCosProperties p, COSClient c) {
        return new QcloudCosTemplate(p, c);
    }
}